Ext.namespace('ds.api');

ds.api.ZoneRecord = Ext.data.Record.create([
    { name: 'id',                   type: 'string' },
    { name: 'name',                 type: 'string' },
    { name: 'isPresent',            type: 'boolean' },
    { name: 'firstZoneOnDSMeter',   type: 'boolean' },
    { name: 'devices' }
]);

ds.api.DeviceRecord = Ext.data.Record.create([
    { name: 'id',           type: 'string' },
    { name: 'isSwitch',     type: 'string' },
    { name: 'functionID',   type: 'int' },
    { name: 'name',         type: 'string' },
    { name: 'on',           type: 'boolean' },
    { name: 'locked',       type: 'boolean' }
]);

ds.api.turnOnOff = function(deviceRecord, state) {
    var verb = (state ? 'turnOn' : 'turnOff'),
        dsid = deviceRecord.get('id');
    Ext.Ajax.request({
        method: 'GET',
        url: '/json/device/' + verb,
        params: {
            dsid: dsid
        },
        success: function(response) {
            if (Ext.decode(response.responseText).ok) {
                deviceRecord.set('on', state);
            }
        }
    });
};

Ext.namespace('ds.ui');

ds.ui.Main = Ext.extend(Ext.Container, {
    initComponent: function() {
        var me = this;
        me.layout = 'anchor';
        me.items = [
            { xtype: 'spacer', height: 10, width: 1 },
            {
                xtype: 'box',
                html: '<h3>Devices</h3>',
            },
            {
                xtype: 'listview',
                itemId: 'deviceView',
                store: me.initialConfig.deviceStore,
                reserveScrollOffset: true,
                autoScroll: true,
                columns: [
                    { header: 'ID',         dataIndex: 'id',    width: 0.3 },
                    { header: 'Enabled',    dataIndex: 'on',    width: 0.1 },
                    { header: 'Name',       dataIndex: 'name' }
                ]
            },
            { xtype: 'spacer', height: 10, width: 1 },
            {
                xtype: 'box',
                html: '<h3>Zones</h3>'
            },
            {
                xtype: 'listview',
                itemId: 'zoneView',
                store: me.initialConfig.zoneStore,
                reserveScrollOffset: true,
                autoScroll: true,
                columns: [
                    { header: 'ID',         dataIndex: 'id',        width: 0.1 },
                    { header: 'Present',    dataIndex: 'isPresent', width: 0.1 },
                    { header: 'name',       dataIndex: 'name' }
                ]
            }
        ];
        ds.ui.Main.superclass.initComponent.call(me);
        me.deviceView = me.getComponent('deviceView');
        me.deviceView.on('click', me.onDeviceViewClick, me);
    },

    onDeviceViewClick: function(view, idx, node, e) {
        var me = this,
            r = view.getRecord(node);
        ds.api.turnOnOff(r, !r.get('on'));
    }
});

Ext.onReady(function() {
    var deviceStore = new Ext.data.JsonStore({
            id: Ext.id(undefined, 'device-'),
            idProperty: 'id',
            fields: ds.api.DeviceRecord
        }),
        zoneStore = new Ext.data.JsonStore({
            id: Ext.id(undefined, 'zone-'),
            idProperty: 'id',
            fields: ds.api.ZoneRecord
        });

    function processStructure(apartment) {
        var zones = [],
            devices = [],
            devDict = [];
        Ext.iterate(apartment.zones, function(zone, zoneId) {
            zones.push(new zoneStore.recordType(zone, zoneId));
            if (zone.id === 0) {
                Ext.iterate(zone.devices, function(dev, devId) {
                    devDict[devId] = dev;
                });
            }
        });
        Ext.iterate(devDict, function(dev, devId) {
            devices.push(new deviceStore.recordType(dev, devId));
        });
        zoneStore.add(zones);
        deviceStore.add(devices);
    }

    function onApartmentStructureLoaded(response) {
        var data = Ext.decode(response.responseText);
        processStructure(data.result.apartment)
        new ds.ui.Main({
            renderTo: 'content',
            width: 620,
            height: 400,
            deviceStore: deviceStore,
            zoneStore: zoneStore
        });
    }

    Ext.Ajax.request({
        url: '/json/apartment/getStructure',
        success: onApartmentStructureLoaded
    });
});
